/* BMW RT11xxRT remote control to Bluetooth adapter version 1.0 by Peter Luijer <peter@luijer.org>
 *   
 * Arduino script for a BlueTooth remote control connected designed to the external radio buttons of a BMW R11xxRT series of motorcycles.
 * The script could also serve other uses, since it's actially nothing more than a programmable BlueTooth BLE keyboard controller.
 * 
 * NOTE: This script has successfully been testedn with "Normal" ESP32's, but somehow on C3 (and possibly also S) -seris of the ESP32 
 * it doesn't work with the current ESP32 Arduino core (BlueTooth is failing after connection has been established).
 * Boards like the NodeMCU and Wemos with "Normal" ESP32, as well as the M5Stamp-Pico work fine (the M5 Stamp C3 and C3U DO NOT).
 * 
 * This script was written with the M5Stamp-Pico in mind; when using another board, then don't forget to change (or disable) 
 * the parts for the LED in the code:
 * - When using a normal ON/OFF LED, just disable the FastLED_H define and set the correct GPIO of the LED for the BT_LED define.
 * - When you don;t want to enable LED support at all, disable FASTLED_H and set BT_LED to -1
 * 
 * Current button assignments:
 * 
 * Button  | Press       | Long press
 * --------+-------------+-----------------------------------------
 * MUTE/TP | Play/pause  | Unlock screen / close active application
 * LEFT    | Prev. track | Start music player
 * UP      | Vol. up     | Take screenshot
 * RIGHT   | Next track  | Start Google maps
 * DOWN    | Vol. down   | Lock screen
 * 
 * 
 * Usage: 
 * Simply make sure everything is connected up, open your phone's BlueTooth settings and connect it to "BMW RT Remote"; that's it!
 * 
 * Extra tip: 
 * Normally you can't fully unlock your phone, but if you add this remote to your "Trusted devices", you can:
 * - On Android: Settings -> Lock Screen -> Smart Lock -> [ENTER PIN] -> Trusted Devices -> Add Trusted Device -> Select "BMW RT Remote" and confirm
 * - On iOS: 
 * 
 */

#define FastLED_H             // Comment out this line, to disable (multi color) FastLED support

#include <EasyButton.h>
#include <BleKeyboard.h>

#ifdef FastLED_H
#include <FastLED.h>
#define NUM_LEDS      1       // Number of LEDs connected (should be 1)
#define BRIGHTNESS    64      // Brightness level of the LED
#define LED_TYPE      SK6812  // Type of multi color LED
#define COLOR_ORDER   GRB     // LED color order
#endif

#define BT_LED        27      // GPIO-pin for the LED (set to -1 to disable)

#define BUTTON_MUTE   18      // GPIO-pin for the Mute button
#define BUTTON_LEFT   19      // GPIO-pin for the Seek prior button
#define BUTTON_UP     21      // GPIO-pin for the Volume up button
#define BUTTON_DOWN   22      // GPIO-pin for the Volume down button
#define BUTTON_RIGHT  25      // GPIO-pin for the Seek next button

#define HOLD_DELAY    1000    // The delay (in ms) to destinguish between short key press and long key press

EasyButton btnMute(BUTTON_MUTE);
EasyButton btnLeft(BUTTON_LEFT);
EasyButton btnUp(BUTTON_UP);
EasyButton btnRight(BUTTON_RIGHT);
EasyButton btnDown(BUTTON_DOWN);

#ifdef FastLED_H
CRGB leds[NUM_LEDS];
#endif

BleKeyboard bleKeyboard("BMW RT Remote", "QuickFix", 100);

int oldConnected = HIGH;

void onMutePressed() {
  Serial.println("Mute short press");
  bleKeyboard.write(KEY_MEDIA_PLAY_PAUSE);
}

void onMuteLongPressed() {
  Serial.println("Mute long press");
  bleKeyboard.write(KEY_ESC); 
}

void onLeftPressed() {
  Serial.println("Left short press");
  bleKeyboard.write(KEY_MEDIA_PREVIOUS_TRACK);
}

void onLeftLongPressed() {  
  Serial.println("Left long press");
  bleKeyboard.press(KEY_LEFT_GUI);
  bleKeyboard.print("p");
  bleKeyboard.releaseAll(); 
}

void onUpPressed() {
  Serial.println("Up short press");
  bleKeyboard.write(KEY_MEDIA_VOLUME_UP);
}

void onUpLongPressed() {
  Serial.println("Up long press");
  bleKeyboard.write(KEY_PRTSC);
}

void onDownPressed() {
  Serial.println("Down short press");
  bleKeyboard.write(KEY_MEDIA_VOLUME_DOWN);
}

void onDownLongPressed() {
  Serial.println("Down long press");
  bleKeyboard.press(KEY_LEFT_GUI);
  bleKeyboard.print("l");
  bleKeyboard.releaseAll();   
}

void onRightPressed() {
  Serial.println("Right short press");
  bleKeyboard.write(KEY_MEDIA_NEXT_TRACK);
}

void onRightLongPressed() {
  Serial.println("Right long press");
  bleKeyboard.press(KEY_LEFT_GUI);
  bleKeyboard.print("m");
  bleKeyboard.releaseAll(); 
}

void setup() {
  Serial.begin(115200); 
  Serial.println("BMW RT Remote starting...");

  if (BT_LED > -1) {
#ifdef FastLED_H
    delay(3000);
    FastLED.addLeds<LED_TYPE, BT_LED, COLOR_ORDER>(leds, NUM_LEDS).setCorrection(TypicalLEDStrip);
    FastLED.setBrightness(BRIGHTNESS);  
    leds[0] = CRGB::Green;
    FastLED.show();
#else
    pinMode(BT_LED, OUTPUT);
    digitalWrite(BT_LED, LOW);
#endif  
  }
    
  btnMute.begin();
  btnMute.onPressed(onMutePressed);
  btnMute.onPressedFor(HOLD_DELAY, onMuteLongPressed);

  btnLeft.begin();
  btnLeft.onPressed(onLeftPressed);
  btnLeft.onPressedFor(HOLD_DELAY, onLeftLongPressed);

  btnUp.begin();
  btnUp.onPressed(onUpPressed);
  btnUp.onPressedFor(HOLD_DELAY, onUpLongPressed);

  btnRight.begin();
  btnRight.onPressed(onRightPressed);
  btnRight.onPressedFor(HOLD_DELAY, onRightLongPressed);

  btnDown.begin();
  btnDown.onPressed(onDownPressed);
  btnDown.onPressedFor(HOLD_DELAY, onDownLongPressed);     
    
  Serial.println("Remote control ready");
  bleKeyboard.begin();     
}

void loop() {
  if (bleKeyboard.isConnected()) {   
    
    if (oldConnected == LOW) {
      oldConnected = HIGH;

      if (BT_LED > -1) {
#ifdef FastLED_H      
        leds[0] = CRGB::Blue;
        FastLED.show(); 
#else
        digitalWrite(BT_LED, oldConnected); 
#endif
      }  
      Serial.println("BlueTooth connected");
    }  
    
    btnMute.read();
    btnLeft.read();
    btnUp.read();
    btnRight.read();
    btnDown.read();    
  }
  else 
  if (oldConnected == HIGH) {
    oldConnected = LOW;    
    
    if (BT_LED > -1) {
#ifdef FastLED_H    
      leds[0] = CRGB::Red;
      FastLED.show(); 
#else   
      digitalWrite(BT_LED, oldConnected);     
#endif
    }
    
    Serial.println("BlueTooth disconnected");
  }     
  yield();
}
